@@ -34,6 +34,7 @@ bit_advance_position_tests (void)
     ok ("bit_advance_position");
   else
     fail ("bit_advance_position");
+  bitfree (&bitchain);
 }
 
 void
@@ -45,6 +46,7 @@ bit_read_B_tests (void)
     pass ();
   else
     fail ("bit_read_B %c", result);
+  bitfree (&bitchain);
 }
 
 void
@@ -57,6 +59,7 @@ bit_write_B_tests (void)
     pass ();
   else
     fail ("bit_write_B");
+  bitfree (&bitchain);
 }
 
 void
@@ -68,6 +71,7 @@ bit_read_BB_tests (void)
     pass ();
   else
     fail ("bit_read_BB %d", result);
+  bitfree (&bitchain);
 }
 
 void
@@ -81,6 +85,7 @@ bit_write_BB_tests (void)
     pass ();
   else
     fail ("bit_write_BB %d", bitchain.chain[0]);
+  bitfree (&bitchain);
 }
 
 void
@@ -102,6 +107,7 @@ bit_read_3B_tests (void)
     pass ();
   else
     fail ("bit_read_3B %d", result);
+  bitfree (&bitchain);
 }
 
 void
@@ -115,6 +121,7 @@ bit_write_3B_tests (void)
     pass ();
   else
     fail ("bit_write_3B %d", bitchain.chain[0]);
+  bitfree (&bitchain);
 }
 
 /* This function calls tests for bit_write_4BITS_tests()
@@ -131,6 +138,7 @@ bit_write_4BITS_tests (void)
     pass ();
   else
     fail ("bit_write_4BITS %x", bitchain.chain[0]);
+  bitfree (&bitchain);
 }
 
 void
@@ -141,7 +149,31 @@ bit_read_4BITS_tests (void)
   if (result == 0xF)
     pass ();
   else
-    fail ("bit_read_4BITS");
+    fail ("bit_read_4BITS 0x%X != 0xF dat:%x", (unsigned)result, bitchain.chain[0]);
+  bitfree (&bitchain);
+
+#define test_4bits(s, x)                                                      \
+  bitchain = strtobt (s);                                                     \
+  result = bit_read_4BITS (&bitchain);                                        \
+  if (result == x)                                                            \
+    pass ();                                                                  \
+  else                                                                        \
+    fail ("bit_read_4BITS 0x%X != 0xF dat:%x", (unsigned)result,              \
+          bitchain.chain[0]);                                                 \
+  bitfree (&bitchain)
+
+  test_4bits ("0000", 0x0);
+  test_4bits ("0001", 0x1);
+  test_4bits ("0010", 0x2);
+  test_4bits ("0011", 0x3);
+  test_4bits ("0100", 0x4);
+  test_4bits ("0101", 0x5);
+  test_4bits ("0111", 0x7);
+  test_4bits ("1000", 0x8);
+  test_4bits ("1001", 0x9);
+  test_4bits ("1100", 0xC);
+  test_4bits ("1101", 0xD);
+  test_4bits ("1111", 0xF);
 }
 
 void
@@ -154,6 +186,7 @@ bit_read_BLL_tests (void)
     pass ();
   else
     fail ("bit_read_BLL " FORMAT_BLL, result);
+  bitfree (&bitchain);
 }
 
 void
@@ -165,6 +198,7 @@ bit_read_RC_tests (void)
     pass ();
   else
     fail ("bit_read_RC");
+  bitfree (&bitchain);
 }
 
 void
@@ -176,6 +210,7 @@ bit_write_RC_tests (void)
     pass ();
   else
     fail ("bit_write_RC");
+  bitfree (&bitchain);
 }
 
 void
@@ -188,6 +223,7 @@ bit_read_RS_tests (void)
     pass ();
   else
     fail ("bit_write_RS");
+  bitfree (&bitchain);
 }
 
 void
@@ -200,6 +236,7 @@ bit_write_RS_tests (void)
     pass ();
   else
     fail ("bit_write_RS");
+  bitfree (&bitchain);
 }
 
 void
@@ -216,6 +253,7 @@ bit_write_RL_tests (void)
   else
     fail ("bit_write_RL %d %d %d %d", bitchain.chain[0], bitchain.chain[1],
           bitchain.chain[2], bitchain.chain[3]);
+  bitfree (&bitchain);
 }
 
 void
@@ -231,6 +269,7 @@ bit_read_RL_tests (void)
     pass ();
   else
     fail ("bit_read_RL 0x%x", (unsigned)result);
+  bitfree (&bitchain);
 }
 
 void
@@ -241,6 +280,7 @@ bit_read_RD_tests (void)
   double result;
 
   bitprepare (&bitchain, sizeof (double));
+  free (bitchain.chain);
   bitchain.chain = (unsigned char *)&value;
   // bit_print (&bitchain, sizeof (double));
 
@@ -268,6 +308,7 @@ bit_write_RD_tests (void)
     pass ();
   else
     fail ("bit_write_RD");
+  bitfree (&bitchain);
 }
 
 int
@@ -282,6 +323,7 @@ main (int argc, char const *argv[])
   BITCODE_UMC umc;
   unsigned long pos;
   long size;
+  char *str;
   unsigned char sentinel[]
       = { 0xCF, 0x7B, 0x1F, 0x23, 0xFD, 0xDE, 0x38, 0xA9,
           0x5F, 0x7C, 0x68, 0xB8, 0x4E, 0x6D, 0x33, 0x5F };
@@ -312,7 +354,9 @@ main (int argc, char const *argv[])
   bitchain.size = 100;
   bitchain.bit = 0;
   bitchain.byte = 0;
-  bitchain.version = R_2000;
+  bitchain.opts = 1;
+  bitchain.fh = NULL;
+  bitchain.version = bitchain.from_version = R_2000;
   bitchain.chain = (unsigned char *)calloc (bitchain.size, 1);
 
   bit_write_RD (&bitchain, 0xBA43B7400);
@@ -321,7 +365,7 @@ main (int argc, char const *argv[])
   else
     fail ("bit_write_RD @%lu.%d", bitchain.byte, bitchain.bit);
 
-  bit_advance_position (&bitchain, -64);
+  bit_advance_position (&bitchain, -64L);
   // bit_print (&bitchain, sizeof (double));
   if ((dbl = bit_read_RD (&bitchain)) == 0xBA43B7400)
     pass ();
@@ -334,13 +378,13 @@ main (int argc, char const *argv[])
   else
     fail ("bit_write_BS @%lu.%d", bitchain.byte, bitchain.bit);
 
-  bit_advance_position (&bitchain, -18);
+  bit_advance_position (&bitchain, -18L);
   // bit_print (&bitchain, 4);
   if ((bs = bit_read_BS (&bitchain)) == 32767)
     pass ();
   else
     fail ("bit_read_BS %d", bs);
-  bit_advance_position (&bitchain, -18);
+  bit_advance_position (&bitchain, -18L);
 
   bit_write_BS (&bitchain, 256);
   if (bitchain.byte == 8 && bitchain.bit == 2)
@@ -348,7 +392,7 @@ main (int argc, char const *argv[])
   else
     fail ("bit_write_BS @%lu.%d", bitchain.byte, bitchain.bit);
 
-  bit_advance_position (&bitchain, -2);
+  bit_advance_position (&bitchain, -2L);
   // bit_print (&bitchain, 1);
   if ((bs = bit_read_BS (&bitchain)) == 256)
     pass ();
@@ -361,7 +405,7 @@ main (int argc, char const *argv[])
   else
     fail ("bit_write_BL @%lu.%d", bitchain.byte, bitchain.bit);
 
-  bit_advance_position (&bitchain, -2);
+  bit_advance_position (&bitchain, -2L);
   // bit_print (&bitchain, 1);
   if ((bl = bit_read_BL (&bitchain)) == 0)
     pass ();
@@ -382,7 +426,7 @@ main (int argc, char const *argv[])
   else
     fail ("bit_read_BD %f", dbl);
 
-  bit_advance_position (&bitchain, 2); // 9.0
+  bit_advance_position (&bitchain, 2L); // 9.0
 
   for (umc = 1; umc <= 0x88000000UL; umc <<= 4)
     {
@@ -428,14 +472,14 @@ main (int argc, char const *argv[])
         break;
     }
 
-  bit_advance_position (&bitchain, 16);
+  bit_advance_position (&bitchain, 16L);
   bit_write_MS (&bitchain, 5000);
   if (bitchain.byte == 13)
     pass ();
   else
     fail ("bit_write_MS @%lu.%d", bitchain.byte, bitchain.bit);
 
-  bit_advance_position (&bitchain, -16);
+  bit_advance_position (&bitchain, -16L);
   if ((bs = bit_read_MS (&bitchain)) == 5000)
     pass ();
   else
@@ -447,7 +491,7 @@ main (int argc, char const *argv[])
   else
     fail ("bit_write_BE @%lu.%d", bitchain.byte, bitchain.bit);
 
-  bit_advance_position (&bitchain, -199);
+  bit_advance_position (&bitchain, -199L);
   {
     double x, y, z;
     bit_read_BE (&bitchain, &x, &y, &z);
@@ -470,7 +514,7 @@ main (int argc, char const *argv[])
   else
     fail ("bit_read_DD %f", dbl);
 
-  bit_advance_position (&bitchain, 64);
+  bit_advance_position (&bitchain, 64L);
   pos = bit_position (&bitchain);
   bit_write_BT (&bitchain, 3.0);
   if (bitchain.bit == 4 && bitchain.byte == 62)
@@ -484,6 +528,10 @@ main (int argc, char const *argv[])
   else
     fail ("bit_read_BT %f", dbl);
 
+  // mingw-w64 gcc-9.2.0 miscompilation in bit_read_H with val[i]: (%rbx) being dat+1
+#if defined(__MINGW64_VERSION_MAJOR) && defined(__GNUC__) && __GNUC__ >= 9
+  if (0)
+#endif
   {
     int i;
     Dwg_Handle handles[]
@@ -514,33 +562,33 @@ main (int argc, char const *argv[])
       }
   }
 #define _CRC 0x6024
-  bit_advance_position (&bitchain, -2);
+  bit_advance_position (&bitchain, -2L);
   {
-    unsigned int crc = bit_write_CRC (&bitchain, 0, 0x64);
+    uint16_t crc = bit_write_CRC (&bitchain, 0UL, 0x64);
     if (crc == _CRC)
       pass ();
     else
       fail ("bit_write_CRC %X", crc);
   }
 
-  bit_advance_position (&bitchain, -16);
-  if (bit_check_CRC (&bitchain, 0, 0x64))
+  bit_advance_position (&bitchain, -16L);
+  if (bit_check_CRC (&bitchain, 0UL, 0x64))
     pass ();
   else
     fail ("bit_check_CRC");
 
-  bit_advance_position (&bitchain, -16);
+  bit_advance_position (&bitchain, -16L);
   if ((bs = bit_read_CRC (&bitchain)) == _CRC)
     pass ();
   else
     fail ("bit_read_CRC %X", bs);
-  bit_advance_position (&bitchain, -16);
+  bit_advance_position (&bitchain, -16L);
 
-  if ((bs = bit_calc_CRC (0x64, (unsigned char *)&bitchain.chain, 0)) == 0x64)
+  if ((bs = bit_calc_CRC (0x64, (unsigned char *)&bitchain.chain, 0L)) == 0x64)
     pass ();
   else
     fail ("bit_calc_CRC %X", bs);
-  bit_advance_position (&bitchain, 16);
+  bit_advance_position (&bitchain, 16L);
 
   bit_write_TV (&bitchain, (char *)"GNU"); // now we store the \0 also
   if (bitchain.byte == 70 && bitchain.bit == 2)
@@ -548,12 +596,14 @@ main (int argc, char const *argv[])
   else
     fail ("bit_write_TV @%ld.%d", bitchain.byte, bitchain.bit);
 
-  bit_advance_position (&bitchain, -42);
-  if (!strcmp ((const char *)bit_read_TV (&bitchain), "GNU"))
+  bit_advance_position (&bitchain, -42L);
+  str = bit_read_TV (&bitchain);
+  if (!strcmp (str, "GNU"))
     pass ();
   else
     fail ("bit_read_TV");
-  bit_advance_position (&bitchain, -8);
+  free (str);
+  bit_advance_position (&bitchain, -8L);
 
   bit_write_L (&bitchain, 20);
   if (bitchain.byte == 73 && bitchain.bit == 2)
@@ -561,7 +611,7 @@ main (int argc, char const *argv[])
   else
     fail ("bit_write_L @%ld.%d", bitchain.byte, bitchain.bit);
 
-  bit_advance_position (&bitchain, -32);
+  bit_advance_position (&bitchain, -32L);
   if ((bl = bit_read_L (&bitchain)) == 20)
     pass ();
   else
@@ -623,6 +673,7 @@ main (int argc, char const *argv[])
     else
       fail ("bit_read_CMC (%d,%u,%d) %s", color.index, color.rgb, color.flag,
             color.name);
+    free (color.name);
   }
 
   bitchain.byte++;
@@ -647,7 +698,7 @@ main (int argc, char const *argv[])
   }
   {
     unsigned int check
-        = bit_calc_CRC (0xC0C1, (unsigned char *)bitchain.chain, 108);
+        = bit_calc_CRC (0xC0C1, (unsigned char *)bitchain.chain, 108L);
     if (check == 0xEB57)
       pass ();
     else
